import logging

from junitparser import JUnitXml, Failure
from tabulate import tabulate

THRESHOLD = 0.05
SLOW_ERROR_MSG = 'Test duration exceeded threshold'
FAIL_TEXT = 'FAIL'


def report_slow_tests(junit_file_path: str, xml_output_file_name: str, display_results: bool):
    """
    Takes a junit test results file and determines if there are any slow tests

    Args:
        junit_file_path (str) : The file path of the junit file results
        xml_output_file_name (str) : Output file name, if a junit file is to be produced. Do not include the file
                                    extension
        display_results (str) : Displays test results if set to true
    """
    junit_xml = JUnitXml.fromfile(junit_file_path)

    test_results, xml_output = parse_test_results(junit_xml)
    average_test_duration = calculate_average_test_duration(junit_xml)

    if display_results:
        print(tabulate(test_results, headers=["Test", "Time (s)", "Result", "Message"]))
        logging.info('Average test duration: %s s', average_test_duration)

    if xml_output_file_name:
        xml_output.write(xml_output_file_name + '.xml')
        logging.info('junit file created: %s.xml', xml_output_file_name)


def parse_test_results(junit_xml: JUnitXml):
    test_results = []

    """
    IF TEST FAILED => FAILED
    IF TEST TOO SLOW => FAILED
    IF TEST TOO SLOW + FAILED => FAILED
    OTHERWISE => PASS
    """

    for suite in junit_xml:
        for testcase in suite:
            if testcase.time > THRESHOLD:
                result = FAIL_TEXT
                if testcase.result and testcase.result[0]:
                    testcase.result[0].message += ' ' + SLOW_ERROR_MSG
                else:
                    testcase.result = [Failure(SLOW_ERROR_MSG)]
                test_results.append([testcase.name, testcase.time, result, testcase.result[0].message])
            elif testcase.result and testcase.result[0]:
                result = FAIL_TEXT
                test_results.append([testcase.name, testcase.time, result, testcase.result[0].message])
            else:
                result = 'PASS'
                test_results.append([testcase.name, testcase.time, result])

    return test_results, junit_xml


def calculate_average_test_duration(junit_xml: JUnitXml) -> float:
    test_count: float = 0
    total_duration: float = 0
    for suite in junit_xml:
        for testcase in suite:
            test_count += 1
            total_duration += testcase.time

    if test_count == 0:
        average = 0
    else:
        average = total_duration / test_count

    return average



import pytest

from slowtestreporter.cli import main


def test_should_throw_error_if_no_path_is_provided():
    with pytest.raises(SystemExit):
        main([])


def test_should_error_when_path_value_that_does_not_exist():
    with pytest.raises(SystemExit):
        main(['dne'])


def test_should_error_when_threshold_is_zero():
    with pytest.raises(SystemExit):
        main(['mytest', '-t', '0'])


def test_should_error_when_threshold_is_negative():
    with pytest.raises(SystemExit):
        main(['myfile', '-t', '-0.7'])


def test_should_error_when_threshold_is_characters():
    with pytest.raises(SystemExit):
        main(['myfile', '-t', 'slow'])



import pytest
from junitparser import TestCase, TestSuite, Error, JUnitXml

from slowtestreporter import slowtestreporter
from slowtestreporter.slowtestreporter import report_slow_tests

# Prevent pytest from trying to collect junitparser test objects as tests:
TestCase.__test__ = False
TestSuite.__test__ = False


def test_should_throw_exception_when_file_is_not_found():
    with pytest.raises(FileNotFoundError):
        report_slow_tests('filedoesnotexist.xml', 'my_results', False)


def test_should_throw_exception_when_no_file_provided():
    with pytest.raises(FileNotFoundError):
        report_slow_tests('', 'results', False)


def test_should_not_change_results_when_single_test_is_fast():
    case1 = TestCase('case1', 'Test', 0.01)
    case1.result = []
    suite = TestSuite('suite1')
    suite.add_testcase(case1)

    xml = JUnitXml()
    xml.add_testsuite(suite)

    test_results, junit_xml = slowtestreporter.parse_test_results(xml)
    assert slowtestreporter.SLOW_ERROR_MSG not in test_results[0], 'Expected no slow test error'


def test_should_not_change_results_when_single_failed_test_is_fast():
    case1 = TestCase('case1', 'FailedTest', 0.01)
    case1.result = [Error('Error', 'Some error type')]
    suite = TestSuite('suite1')
    suite.add_testcase(case1)

    xml = JUnitXml()
    xml.add_testsuite(suite)

    test_results, junit_xml = slowtestreporter.parse_test_results(xml)
    assert slowtestreporter.SLOW_ERROR_MSG not in test_results[0], 'Expected no slow test error despite failed test'


def test_should_keep_test_failed_when_single_failed_test_is_fast():
    case1 = TestCase('case1', 'FailedTest', 0.01)
    case1.result = [Error('Error', 'Some error type')]
    suite = TestSuite('suite1')
    suite.add_testcase(case1)

    xml = JUnitXml()
    xml.add_testsuite(suite)

    test_results, junit_xml = slowtestreporter.parse_test_results(xml)
    assert slowtestreporter.FAIL_TEXT in test_results[0][2], 'Expected failed test to stay failed'


def test_should_report_slow_test_when_single_failed_test_is_slow():
    case1 = TestCase('case1', 'FailedTest', 7000)
    case1.result = [Error('Error', 'Some error type')]
    suite = TestSuite('suite1')
    suite.add_testcase(case1)

    xml = JUnitXml()
    xml.add_testsuite(suite)

    test_results, junit_xml = slowtestreporter.parse_test_results(xml)
    assert slowtestreporter.FAIL_TEXT in test_results[0][2], 'Expected slow test error for failed and slow test'


def test_should_not_fail_average_calculation_when_no_test_suite():
    xml = JUnitXml()

    average_time = slowtestreporter.calculate_average_test_duration(xml)
    assert average_time == 0, 'Expected no failure when there is no test suite'


def test_should_calculate_average_time_for_single_test():
    case1 = TestCase('case1', 'Test', 0.01)
    case1.result = []
    suite = TestSuite('suite1')
    suite.add_testcase(case1)

    xml = JUnitXml()
    xml.add_testsuite(suite)

    average_time = slowtestreporter.calculate_average_test_duration(xml)
    assert average_time == 0.01, 'Expected correct average test duration when calculated against single test case'


def test_should_calculate_average_time_for_two_test_cases():
    case1 = TestCase('case1', 'Test', 0.01)
    case1.result = []

    case2 = TestCase('case2', 'Test', 0.12)
    case2.result = []

    suite = TestSuite('suite1')
    suite.add_testcase(case1)
    suite.add_testcase(case2)

    xml = JUnitXml()
    xml.add_testsuite(suite)

    average_time = slowtestreporter.calculate_average_test_duration(xml)
    assert average_time == 0.065, 'Expected correct average test duration when calculated against 2 test cases'
